@@ -37,25 +37,12 @@ fi
 # try to detect NDK version
 REL=$(grep -o '^r[0-9]*.*' $ANDROID_NDK/RELEASE.TXT 2>/dev/null|cut -b2-)
 case "$REL" in
-    9*)
+    9*|10*)
         GCCVER=4.8
         CXXSTL="/"${GCCVER}
     ;;
-    8?*)
-        # we don't use 4.4.3 because it doesn't handle threads correctly.
-        # TODO : clang?
-        if test -d ${ANDROID_NDK}/toolchains/arm-linux-androideabi-4.7
-        # if gcc 4.7 is present, it's there for all the archs (x86, mips, arm)
-        then
-            # since r8d
-            GCCVER=4.7
-        else
-            GCCVER=4.6
-        fi
-        CXXSTL="/"${GCCVER}
-    ;;
     7|8|*)
-        echo "You need the NDKv8b or later"
+        echo "You need the NDKv9 or later"
         exit 1
     ;;
 esac
@@ -105,13 +92,14 @@ export PATH=${NDK_TOOLCHAIN_PATH}:${PATH}
 
 ANDROID_PATH="`pwd`"
 
+# Fetch VLC source
 if [ ! -z "$FETCH" ]
 then
     # 1/ libvlc, libvlccore and its plugins
-    TESTED_HASH=0b4c9bc
+    TESTED_HASH=036010e
     if [ ! -d "vlc" ]; then
         echo "VLC source not found, cloning"
-        git clone git://git.videolan.org/vlc.git vlc
+        git clone git://git.videolan.org/vlc/vlc-2.2.git vlc
         cd vlc
         git checkout -B android ${TESTED_HASH}
     else
@@ -146,6 +134,7 @@ then
     exit 0
 fi
 
+# Setup CFLAGS
 if [ ${ANDROID_ABI} = "armeabi-v7a" ] ; then
     EXTRA_CFLAGS="-mfpu=vfpv3-d16 -mcpu=cortex-a8"
     EXTRA_CFLAGS="${EXTRA_CFLAGS} -mthumb -mfloat-abi=softfp"
@@ -160,7 +149,7 @@ elif [ ${ANDROID_ABI} = "armeabi" ] ; then
         fi
     fi
 elif [ ${ANDROID_ABI} = "x86" ] ; then
-    EXTRA_CFLAGS="-march=pentium"
+    EXTRA_CFLAGS="-march=pentium -m32"
 elif [ ${ANDROID_ABI} = "mips" ] ; then
     EXTRA_CFLAGS="-march=mips32 -mtune=mips32r2 -mhard-float"
     # All MIPS Linux kernels since 2.4.4 will trap any unimplemented FPU
@@ -176,6 +165,10 @@ EXTRA_CFLAGS="${EXTRA_CFLAGS} -O2"
 EXTRA_CFLAGS="${EXTRA_CFLAGS} -I${ANDROID_NDK}/sources/cxx-stl/gnu-libstdc++${CXXSTL}/include"
 EXTRA_CFLAGS="${EXTRA_CFLAGS} -I${ANDROID_NDK}/sources/cxx-stl/gnu-libstdc++${CXXSTL}/libs/${ANDROID_ABI}/include"
 
+# Setup LDFLAGS
+EXTRA_LDFLAGS="-l${ANDROID_NDK}/sources/cxx-stl/gnu-libstdc++${CXXSTL}/libs/${ANDROID_ABI}/libgnustl_static.a"
+
+# Make in //
 UNAMES=$(uname -s)
 MAKEFLAGS=
 if which nproc >/dev/null
@@ -186,6 +179,7 @@ then
 MAKEFLAGS=-j`sysctl -n machdep.cpu.thread_count`
 fi
 
+# Build buildsystem tools
 export PATH=`pwd`/extras/tools/build/bin:$PATH
 echo "Building tools"
 cd extras/tools
@@ -193,8 +187,11 @@ cd extras/tools
 make $MAKEFLAGS
 cd ../..
 
+############
+# Contribs #
+############
 echo "Building the contribs"
-mkdir -p contrib/android
+mkdir -p contrib/contrib-android-${TARGET_TUPLE}
 
 gen_pc_file() {
     echo "Generating $1 pkg-config file"
@@ -209,9 +206,10 @@ mkdir -p contrib/${TARGET_TUPLE}/lib/pkgconfig
 gen_pc_file EGL 1.1
 gen_pc_file GLESv2 2
 
-cd contrib/android
+cd contrib/contrib-android-${TARGET_TUPLE}
 ../bootstrap --host=${TARGET_TUPLE} --disable-disc --disable-sout \
     --enable-dvdread \
+    --enable-dvdnav \
     --disable-dca \
     --disable-goom \
     --disable-chromaprint \
@@ -233,7 +231,8 @@ cd contrib/android
     --disable-samplerate \
     --disable-faad2 \
     --disable-harfbuzz \
-    --enable-iconv
+    --enable-iconv \
+    --disable-aribb24
 
 # TODO: mpeg2, theora
 
@@ -252,7 +251,9 @@ else
 fi
 
 echo "EXTRA_CFLAGS= -g ${EXTRA_CFLAGS}" >> config.mak
+echo "EXTRA_LDFLAGS= ${EXTRA_LDFLAGS}" >> config.mak
 export VLC_EXTRA_CFLAGS="${EXTRA_CFLAGS}"
+export VLC_EXTRA_LDFLAGS="${EXTRA_LDFLAGS}"
 
 make fetch
 # We already have zlib available
@@ -261,40 +262,50 @@ which autopoint >/dev/null || make $MAKEFLAGS .gettext
 export PATH="$PATH:$PWD/../$TARGET_TUPLE/bin"
 make $MAKEFLAGS
 
-cd ../.. && mkdir -p android && cd android
+############
+# Make VLC #
+############
+cd ../.. && mkdir -p build-android-${TARGET_TUPLE} && cd build-android-${TARGET_TUPLE}
 
 if [ $# -eq 1 ] && [ "$1" = "jni" ]; then
     CLEAN="jniclean"
     TARGET="vlc-android/obj/local/armeabi-v7a/libvlcjni.so"
 else
     CLEAN="distclean"
-    echo "Bootstraping"
-    ../bootstrap
-    echo "Configuring"
-    ${ANDROID_PATH}/configure.sh $OPTS
+    if [ ! -f config.h ]; then
+        echo "Bootstraping"
+        ../bootstrap
+        echo "Configuring"
+        ${ANDROID_PATH}/configure.sh $OPTS
+    fi
     TARGET=
 fi
 
 echo "Building"
 make $MAKEFLAGS
 
-
-# 2/ VLC android UI and specific code
-
-echo "Building Android"
+####################################
+# VLC android UI and specific code
+####################################
+echo "Building VLC for Android"
 cd ../../
 
 export ANDROID_SYS_HEADERS_GINGERBREAD=${PWD}/android-headers-gingerbread
 export ANDROID_SYS_HEADERS_HC=${PWD}/android-headers-hc
 export ANDROID_SYS_HEADERS_ICS=${PWD}/android-headers-ics
+export ANDROID_SYS_HEADERS_JB=${PWD}/android-headers-jb
+export ANDROID_SYS_HEADERS_JBMR2=${PWD}/android-headers-jbmr2
+export ANDROID_SYS_HEADERS_KK=${PWD}/android-headers-kk
 
 export ANDROID_LIBS=${PWD}/android-libs
-export VLC_BUILD_DIR=vlc/android
+export VLC_BUILD_DIR=vlc/build-android-${TARGET_TUPLE}
 
 make $CLEAN
 make -j1 TARGET_TUPLE=$TARGET_TUPLE PLATFORM_SHORT_ARCH=$PLATFORM_SHORT_ARCH CXXSTL=$CXXSTL RELEASE=$RELEASE $TARGET
 
-# 3/ Environment script
+#
+# Exporting a environment script with all the necessary variables
+#
 echo "Generating environment script."
 cat <<EOF
 This is a script that will export many of the variables used in this
@@ -326,8 +337,11 @@ export CXXSTL=$CXXSTL
 export ANDROID_SYS_HEADERS_GINGERBREAD=$ANDROID_SYS_HEADERS_GINGERBREAD
 export ANDROID_SYS_HEADERS_HC=$ANDROID_SYS_HEADERS_HC
 export ANDROID_SYS_HEADERS_ICS=$ANDROID_SYS_HEADERS_ICS
+export ANDROID_SYS_HEADERS_JB=$ANDROID_SYS_HEADERS_JB
+export ANDROID_SYS_HEADERS_JBMR2=$ANDROID_SYS_HEADERS_JBMR2
+export ANDROID_SYS_HEADERS_KK=$ANDROID_SYS_HEADERS_KK
 export ANDROID_LIBS=$ANDROID_LIBS
-export VLC_BUILD_DIR=$PWD/vlc/android
+export VLC_BUILD_DIR=$VLC_BUILD_DIR
 export TARGET_TUPLE=$TARGET_TUPLE
 export PATH_HOST=$PATH_HOST
 export PLATFORM_SHORT_ARCH=$PLATFORM_SHORT_ARCH
