@@ -1,7 +1,7 @@
 /*****************************************************************************
  * EventHandler.java
  *****************************************************************************
- * Copyright © 2011-2012 VLC authors and VideoLAN
+ * Copyright © 2011-2014 VLC authors and VideoLAN
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published by
@@ -35,7 +35,7 @@ public class EventHandler {
     //public static final int MediaMetaChanged                = 0;
     //public static final int MediaSubItemAdded               = 1;
     //public static final int MediaDurationChanged            = 2;
-    //public static final int MediaParsedChanged              = 3;
+    public static final int MediaParsedChanged                = 3;
     //public static final int MediaFreed                      = 4;
     //public static final int MediaStateChanged               = 5;
 
@@ -50,7 +50,7 @@ public class EventHandler {
     //public static final int MediaPlayerBackward             = 0x108;
     public static final int MediaPlayerEndReached             = 0x109;
     public static final int MediaPlayerEncounteredError       = 0x10a;
-    //public static final int MediaPlayerTimeChanged          = 0x10b;
+    public static final int MediaPlayerTimeChanged            = 0x10b;
     public static final int MediaPlayerPositionChanged        = 0x10c;
     //public static final int MediaPlayerSeekableChanged      = 0x10d;
     //public static final int MediaPlayerPausableChanged      = 0x10e;
@@ -59,9 +59,9 @@ public class EventHandler {
     //public static final int MediaPlayerLengthChanged        = 0x111;
     public static final int MediaPlayerVout                   = 0x112;
 
-    public static final int MediaListItemAdded                = 0x200;
+    //public static final int MediaListItemAdded              = 0x200;
     //public static final int MediaListWillAddItem            = 0x201;
-    public static final int MediaListItemDeleted              = 0x202;
+    //public static final int MediaListItemDeleted            = 0x202;
     //public static final int MediaListWillDeleteItem         = 0x203;
 
     //public static final int MediaListViewItemAdded          = 0x300;
@@ -88,10 +88,18 @@ public class EventHandler {
     //public static final int VlmMediaInstanceStatusEnd       = 0x609;
     //public static final int VlmMediaInstanceStatusError     = 0x60a;
 
+    public static final int CustomMediaListExpanding          = 0x2000;
+    public static final int CustomMediaListExpandingEnd       = 0x2001;
+    public static final int CustomMediaListItemAdded          = 0x2002;
+    public static final int CustomMediaListItemDeleted        = 0x2003;
+    public static final int CustomMediaListItemMoved          = 0x2004;
+
+    public static final int HardwareAccelerationError         = 0x3000;
+
     private ArrayList<Handler> mEventHandler;
     private static EventHandler mInstance;
 
-    private EventHandler() {
+    EventHandler() {
         mEventHandler = new ArrayList<Handler>();
     }
 
