@@ -19,6 +19,11 @@
  *****************************************************************************/
 package org.videolan.vlc;
 
+import org.videolan.libvlc.LibVLC;
+import org.videolan.libvlc.LibVlcException;
+import org.videolan.vlc.audio.AudioService;
+import org.videolan.vlc.util.VLCInstance;
+
 import android.content.BroadcastReceiver;
 import android.content.Context;
 import android.content.Intent;
@@ -41,7 +46,7 @@ public class RemoteControlClientReceiver extends BroadcastReceiver {
         String action = intent.getAction();
         LibVLC mLibVLC;
         try {
-            mLibVLC = LibVLC.getInstance();
+            mLibVLC = VLCInstance.getLibVlcInstance();
         } catch (LibVlcException e) {
             return;
         }
@@ -118,6 +123,10 @@ public class RemoteControlClientReceiver extends BroadcastReceiver {
                 abortBroadcast();
             if(i != null)
                 context.sendBroadcast(i);
+        } else if (action.equals(AudioService.ACTION_REMOTE_PLAYPAUSE)){
+            intent = new Intent(context, AudioService.class);
+            intent.setAction(AudioService.ACTION_REMOTE_PLAYPAUSE);
+            context.startService(intent);
         }
     }
 }
