@@ -22,6 +22,7 @@ package org.videolan.vlc;
 import java.util.Locale;
 
 import org.videolan.vlc.gui.audio.AudioUtil;
+import org.videolan.vlc.util.BitmapCache;
 
 import android.app.Application;
 import android.content.Context;
@@ -36,6 +37,8 @@ public class VLCApplication extends Application {
     private static VLCApplication instance;
 
     public final static String SLEEP_INTENT = "org.videolan.vlc.SleepIntent";
+    public final static String INCOMING_CALL_INTENT = "org.videolan.vlc.IncomingCallIntent";
+    public final static String CALL_ENDED_INTENT = "org.videolan.vlc.CallEndedIntent";
 
     @Override
     public void onCreate() {
@@ -75,7 +78,7 @@ public class VLCApplication extends Application {
         instance = this;
 
         // Initialize the database soon enough to avoid any race condition and crash
-        MediaDatabase.getInstance(this);
+        MediaDatabase.getInstance();
         // Prepare cache folder constants
         AudioUtil.prepareCacheFolder(this);
     }
