@@ -30,7 +30,9 @@ import java.util.Stack;
 import org.videolan.libvlc.Media;
 import org.videolan.vlc.MediaDatabase;
 import org.videolan.vlc.R;
-import org.videolan.vlc.Util;
+import org.videolan.vlc.util.AndroidDevices;
+import org.videolan.vlc.util.CustomDirectories;
+import org.videolan.vlc.util.Util;
 
 import android.app.AlertDialog;
 import android.app.ListActivity;
@@ -49,6 +51,7 @@ import android.view.View;
 import android.widget.AdapterView.AdapterContextMenuInfo;
 import android.widget.EditText;
 import android.widget.ListView;
+import android.widget.Toast;
 
 public class BrowserActivity extends ListActivity {
     public final static String TAG = "VLC/BrowserActivity";
@@ -62,7 +65,7 @@ public class BrowserActivity extends ListActivity {
     private final Stack<ScrollState> mScrollStates = new Stack<ScrollState>();
     private String mRoots[];
 
-    private class ScrollState {
+    private static class ScrollState {
         public ScrollState(int index, int top) {
             this.index = index;
             this.top = top;
@@ -95,8 +98,8 @@ public class BrowserActivity extends ListActivity {
 
     private void refreshRoots() {
         ArrayList<String> list = new ArrayList<String>();
-        list.addAll(Arrays.asList(Util.getStorageDirectories()));
-        list.addAll(Arrays.asList(Util.getCustomDirectories()));
+        list.addAll(Arrays.asList(AndroidDevices.getStorageDirectories()));
+        list.addAll(Arrays.asList(CustomDirectories.getCustomDirectories()));
         mRoots = list.toArray(new String[list.size()]);
     }
 
@@ -114,7 +117,7 @@ public class BrowserActivity extends ListActivity {
         final File item = mAdapter.getItem(position);
         if (mCurrentDir != null
                 || item.getPath().equals(BrowserAdapter.ADD_ITEM_PATH)
-                || Arrays.asList(Util.getStorageDirectories()).contains(
+                || Arrays.asList(AndroidDevices.getStorageDirectories()).contains(
                         item.getPath())) {
             return;
         }
@@ -124,12 +127,12 @@ public class BrowserActivity extends ListActivity {
             @Override
             public boolean onMenuItemClick(MenuItem arg0) {
                 // remove any checkmarks of the custom item
-                final MediaDatabase dbManager = MediaDatabase.getInstance(BrowserActivity.this);
+                final MediaDatabase dbManager = MediaDatabase.getInstance();
                 for(File f : dbManager.getMediaDirs()) {
                     if(f.getPath().startsWith(item.getPath()))
                         dbManager.removeDir(f.getPath());
                 }
-                Util.removeCustomDirectory(item.getPath());
+                CustomDirectories.removeCustomDirectory(item.getPath());
                 refresh();
                 return true;
             }
@@ -190,7 +193,14 @@ public class BrowserActivity extends ListActivity {
             b.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
                 @Override
                 public void onClick(DialogInterface dialog, int which) {
-                    Util.addCustomDirectory(input.getText().toString());
+                    String path = input.getText().toString().trim();
+                    File f = new File(path);
+                    if(!f.exists() || !f.isDirectory()) {
+                        Toast.makeText(BrowserActivity.this, getString(R.string.directorynotfound, path), Toast.LENGTH_SHORT).show();
+                        return;
+                    }
+
+                    CustomDirectories.addCustomDirectory(f.getAbsolutePath());
                     refresh();
                 }
             });
