@@ -26,14 +26,13 @@ import java.util.List;
 
 import org.videolan.libvlc.LibVlcException;
 import org.videolan.libvlc.LibVlcUtil;
-import org.videolan.vlc.AudioService;
-import org.videolan.vlc.AudioServiceController;
 import org.videolan.vlc.MediaDatabase;
 import org.videolan.vlc.MediaLibrary;
 import org.videolan.vlc.R;
-import org.videolan.vlc.Util;
+import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.VLCCallbackTask;
-import org.videolan.vlc.WeakHandler;
+import org.videolan.vlc.audio.AudioService;
+import org.videolan.vlc.audio.AudioServiceController;
 import org.videolan.vlc.gui.SidebarAdapter.SidebarEntry;
 import org.videolan.vlc.gui.audio.AudioAlbumsSongsFragment;
 import org.videolan.vlc.gui.audio.AudioPlayer;
@@ -41,12 +40,15 @@ import org.videolan.vlc.gui.audio.EqualizerFragment;
 import org.videolan.vlc.gui.video.MediaInfoFragment;
 import org.videolan.vlc.gui.video.VideoGridFragment;
 import org.videolan.vlc.gui.video.VideoListAdapter;
+import org.videolan.vlc.interfaces.IRefreshable;
 import org.videolan.vlc.interfaces.ISortable;
+import org.videolan.vlc.util.Util;
+import org.videolan.vlc.util.VLCInstance;
+import org.videolan.vlc.util.WeakHandler;
 import org.videolan.vlc.widget.SlidingPaneLayout;
 
 import android.annotation.TargetApi;
 import android.app.AlertDialog;
-import android.app.Dialog;
 import android.content.BroadcastReceiver;
 import android.content.Context;
 import android.content.DialogInterface;
@@ -62,35 +64,33 @@ import android.os.Bundle;
 import android.os.Handler;
 import android.os.Message;
 import android.preference.PreferenceManager;
+import android.support.v4.app.ActionBarDrawerToggle;
 import android.support.v4.app.Fragment;
 import android.support.v4.app.FragmentActivity;
 import android.support.v4.app.FragmentManager;
 import android.support.v4.app.FragmentTransaction;
+import android.support.v4.view.GravityCompat;
+import android.support.v4.widget.DrawerLayout;
+import android.support.v7.app.ActionBar;
+import android.support.v7.app.ActionBarActivity;
 import android.text.InputType;
 import android.util.Log;
-import android.view.Display;
+import android.view.KeyEvent;
 import android.view.LayoutInflater;
+import android.view.Menu;
+import android.view.MenuInflater;
+import android.view.MenuItem;
 import android.view.View;
-import android.view.View.OnClickListener;
+import android.view.Window;
 import android.view.WindowManager;
 import android.widget.AdapterView;
 import android.widget.AdapterView.OnItemClickListener;
-import android.widget.Button;
-import android.widget.CheckBox;
 import android.widget.EditText;
 import android.widget.ListView;
 import android.widget.ProgressBar;
 import android.widget.TextView;
 
-import com.actionbarsherlock.app.ActionBar;
-import com.actionbarsherlock.app.SherlockFragmentActivity;
-import com.actionbarsherlock.view.Menu;
-import com.actionbarsherlock.view.MenuInflater;
-import com.actionbarsherlock.view.MenuItem;
-import com.actionbarsherlock.view.Window;
-import com.slidingmenu.lib.SlidingMenu;
-
-public class MainActivity extends SherlockFragmentActivity {
+public class MainActivity extends ActionBarActivity {
     public final static String TAG = "VLC/MainActivity";
 
     protected static final String ACTION_SHOW_PROGRESSBAR = "org.videolan.vlc.gui.ShowProgressBar";
@@ -105,11 +105,13 @@ public class MainActivity extends SherlockFragmentActivity {
     private static final int ACTIVITY_SHOW_INFOLAYOUT = 2;
 
     private ActionBar mActionBar;
-    private SlidingMenu mMenu;
     private SidebarAdapter mSidebarAdapter;
     private AudioPlayer mAudioPlayer;
     private AudioServiceController mAudioController;
     private SlidingPaneLayout mSlidingPane;
+    private DrawerLayout mRootContainer;
+    private ListView mListView;
+    private ActionBarDrawerToggle mDrawerToggle;
 
     private View mInfoLayout;
     private ProgressBar mInfoProgress;
@@ -129,6 +131,7 @@ public class MainActivity extends SherlockFragmentActivity {
     private boolean mScanNeeded = true;
 
     private Handler mHandler = new MainActivityHandler(this);
+    private int mFocusedPrior = 0;
 
     @Override
     protected void onCreate(Bundle savedInstanceState) {
@@ -164,7 +167,7 @@ public class MainActivity extends SherlockFragmentActivity {
 
         try {
             // Start LibVLC
-            Util.getLibVlcInstance();
+            VLCInstance.getLibVlcInstance();
         } catch (LibVlcException e) {
             e.printStackTrace();
             Intent i = new Intent(this, CompatErrorActivity.class);
@@ -176,6 +179,10 @@ public class MainActivity extends SherlockFragmentActivity {
             return;
         }
 
+        /* Load media items from database and storage */
+        if (mScanNeeded)
+            MediaLibrary.getInstance().loadMediaItems();
+
         super.onCreate(savedInstanceState);
 
         /*** Start initializing the UI ***/
@@ -183,10 +190,6 @@ public class MainActivity extends SherlockFragmentActivity {
         /* Enable the indeterminate progress feature */
         requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
 
-        // Set up the sliding menu
-        mMenu = (SlidingMenu) LayoutInflater.from(this).inflate(R.layout.sliding_menu, null);
-        changeMenuOffset();
-
         SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(this);
         boolean enableBlackTheme = pref.getBoolean("enable_black_theme", false);
         if (enableBlackTheme)
@@ -198,34 +201,46 @@ public class MainActivity extends SherlockFragmentActivity {
         mSlidingPane = (SlidingPaneLayout) v_main.findViewById(R.id.pane);
         mSlidingPane.setPanelSlideListener(mPanelSlideListener);
 
-        View sidebar = LayoutInflater.from(this).inflate(R.layout.sidebar, null);
-        final ListView listView = (ListView)sidebar.findViewById(android.R.id.list);
-        listView.setFooterDividersEnabled(true);
+        mListView = (ListView)v_main.findViewById(R.id.sidelist);
+        mListView.setFooterDividersEnabled(true);
         mSidebarAdapter = new SidebarAdapter(this);
-        listView.setAdapter(mSidebarAdapter);
-        mMenu.setMenu(sidebar);
-        mMenu.attachToActivity(this, SlidingMenu.SLIDING_CONTENT, true);
+        mListView.setAdapter(mSidebarAdapter);
+
 
         /* Initialize UI variables */
         mInfoLayout = v_main.findViewById(R.id.info_layout);
         mInfoProgress = (ProgressBar) v_main.findViewById(R.id.info_progress);
         mInfoText = (TextView) v_main.findViewById(R.id.info_text);
         mAudioPlayerFilling = v_main.findViewById(R.id.audio_player_filling);
+        mRootContainer = (DrawerLayout) v_main.findViewById(R.id.root_container);
 
         /* Set up the action bar */
         prepareActionBar();
 
-        /* Set up the sidebar click listener */
-        listView.setOnItemClickListener(new OnItemClickListener() {
+        /* Set up the sidebar click listener
+         * no need to invalidate menu for now */
+        mDrawerToggle = new ActionBarDrawerToggle(this, mRootContainer,
+                R.drawable.ic_drawer, R.string.drawer_open, R.string.drawer_close) {};
+
+        // Set the drawer toggle as the DrawerListener
+        mRootContainer.setDrawerListener(mDrawerToggle);
+        // set a custom shadow that overlays the main content when the drawer opens
+        mRootContainer.setDrawerShadow(R.drawable.drawer_shadow, GravityCompat.START);
+        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
+        getSupportActionBar().setHomeButtonEnabled(true);
+
+        mListView.setOnItemClickListener(new OnItemClickListener() {
 
             @Override
             public void onItemClick(AdapterView<?> parent, View view,
                     int position, long id) {
-                SidebarAdapter.SidebarEntry entry = (SidebarEntry) listView.getItemAtPosition(position);
+                SidebarAdapter.SidebarEntry entry = (SidebarEntry) mListView.getItemAtPosition(position);
                 Fragment current = getSupportFragmentManager().findFragmentById(R.id.fragment_placeholder);
 
-                if(current == null || current.getTag().equals(entry.id)) { /* Already selected */
-                    mMenu.showContent();
+                if(current == null || (entry != null && current.getTag().equals(entry.id))) { /* Already selected */
+                    if (mFocusedPrior != 0)
+                        findViewById(R.id.ml_menu_search).requestFocus();
+                    mRootContainer.closeDrawer(mListView);
                     return;
                 }
 
@@ -259,7 +274,9 @@ public class MainActivity extends SherlockFragmentActivity {
                 if(current.getTag().equals("tracks"))
                     getFragment("audio").setUserVisibleHint(false);
 
-                mMenu.showContent();
+                if (mFocusedPrior != 0)
+                    findViewById(R.id.ml_menu_search).requestFocus();
+                mRootContainer.closeDrawer(mListView);
             }
         });
 
@@ -271,10 +288,7 @@ public class MainActivity extends SherlockFragmentActivity {
             .replace(R.id.audio_player, mAudioPlayer)
             .commit();
 
-        /* Show info/alpha/beta Warning */
-        if (mSettings.getInt(PREF_SHOW_INFO, -1) != mVersionNumber)
-            showInfoDialog();
-        else if (mFirstRun) {
+        if (mFirstRun) {
             /*
              * The sliding menu is automatically opened when the user closes
              * the info dialog. If (for any reason) the dialog is not shown,
@@ -283,7 +297,7 @@ public class MainActivity extends SherlockFragmentActivity {
             mHandler.postDelayed(new Runnable() {
                 @Override
                 public void run() {
-                    mMenu.showMenu();
+                    mRootContainer.openDrawer(mListView);
                 }
             }, 500);
         }
@@ -300,12 +314,12 @@ public class MainActivity extends SherlockFragmentActivity {
         reloadPreferences();
     }
 
-    private void changeMenuOffset() {
-        WindowManager wm = (WindowManager) getSystemService(WINDOW_SERVICE);
-        Display display = wm.getDefaultDisplay();
-        @SuppressWarnings("deprecation")
-        int behindOffset_dp = Util.convertPxToDp(display.getWidth()) - 208;
-        mMenu.setBehindOffset(Util.convertDpToPx(behindOffset_dp));
+
+    @Override
+    protected void onPostCreate(Bundle savedInstanceState) {
+        super.onPostCreate(savedInstanceState);
+        // Sync the toggle state after onRestoreInstanceState has occurred.
+        mDrawerToggle.syncState();
     }
 
     @TargetApi(Build.VERSION_CODES.HONEYCOMB)
@@ -324,11 +338,7 @@ public class MainActivity extends SherlockFragmentActivity {
         /* FIXME: this is used to avoid having MainActivity twice in the backstack */
         if (getIntent().hasExtra(AudioService.START_FROM_NOTIFICATION))
             getIntent().removeExtra(AudioService.START_FROM_NOTIFICATION);
-
-        /* Load media items from database and storage */
-        if (mScanNeeded)
-            MediaLibrary.getInstance(this).loadMediaItems(this);
-    }
+   }
 
     @Override
     protected void onResumeFragments() {
@@ -339,12 +349,7 @@ public class MainActivity extends SherlockFragmentActivity {
                 .findFragmentById(R.id.fragment_placeholder);
         boolean found = false;
         if(current != null) {
-            for(int i = 0; i < SidebarAdapter.entries.size(); i++) {
-                if(SidebarAdapter.entries.get(i).id.equals(current.getTag())) {
-                    found = true;
-                    break;
-                }
-            }
+            found = SidebarAdapter.sidebarFragments.contains(current.getTag());
         } else {
             found = true;
         }
@@ -377,8 +382,12 @@ public class MainActivity extends SherlockFragmentActivity {
          */
         if(current == null || (!current.getTag().equals(mCurrentFragment) && found)) {
             Log.d(TAG, "Reloading displayed fragment");
-            if (mCurrentFragment == null || secondaryFragments.contains(mCurrentFragment))
+            if(mCurrentFragment == null || secondaryFragments.contains(mCurrentFragment))
                 mCurrentFragment = "video";
+            if(!SidebarAdapter.sidebarFragments.contains(mCurrentFragment)) {
+                Log.d(TAG, "Unknown fragment \"" + mCurrentFragment + "\", resetting to video");
+                mCurrentFragment = "video";
+            }
             Fragment ff = getFragment(mCurrentFragment);
             FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
             ft.replace(R.id.fragment_placeholder, ff, mCurrentFragment);
@@ -394,9 +403,9 @@ public class MainActivity extends SherlockFragmentActivity {
         super.onPause();
 
         /* Check for an ongoing scan that needs to be resumed during onResume */
-        mScanNeeded = MediaLibrary.getInstance(this).isWorking();
+        mScanNeeded = MediaLibrary.getInstance().isWorking();
         /* Stop scanning for files */
-        MediaLibrary.getInstance(this).stop();
+        MediaLibrary.getInstance().stop();
         /* Save the tab status in pref */
         SharedPreferences.Editor editor = getSharedPreferences("MainActivity", MODE_PRIVATE).edit();
         editor.putString("fragment", mCurrentFragment);
@@ -404,6 +413,7 @@ public class MainActivity extends SherlockFragmentActivity {
 
         mAudioController.removeAudioPlayer(mAudioPlayer);
         AudioServiceController.getInstance().unbindAudioService(this);
+        mFocusedPrior = 0;
     }
 
     @Override
@@ -424,9 +434,11 @@ public class MainActivity extends SherlockFragmentActivity {
 
     @Override
     public void onBackPressed() {
-        if(mMenu.isMenuShowing()) {
+        if(mRootContainer.isDrawerOpen(mListView)) {
             /* Close the menu first */
-            mMenu.showContent();
+            if (mFocusedPrior != 0)
+                findViewById(R.id.ml_menu_search).requestFocus();
+            mRootContainer.closeDrawer(mListView);
             return;
         }
 
@@ -555,7 +567,7 @@ public class MainActivity extends SherlockFragmentActivity {
          * is called while the view is created. This can happen
          * any time after onCreate.
          */
-        MenuInflater inflater = getSupportMenuInflater();
+        MenuInflater inflater = getMenuInflater();
         inflater.inflate(R.menu.media_library, menu);
         return super.onCreateOptionsMenu(menu);
     }
@@ -581,7 +593,7 @@ public class MainActivity extends SherlockFragmentActivity {
     @Override
     public void onConfigurationChanged(Configuration newConfig) {
         super.onConfigurationChanged(newConfig);
-        changeMenuOffset();
+        mDrawerToggle.onConfigurationChanged(newConfig);
     }
 
     @Override
@@ -600,12 +612,13 @@ public class MainActivity extends SherlockFragmentActivity {
 
         // Intent to start a new Activity
         Intent intent;
+        // Current fragment loaded
+        Fragment current = getSupportFragmentManager().findFragmentById(R.id.fragment_placeholder);
 
         // Handle item selection
         switch (item.getItemId()) {
             case R.id.ml_menu_sortby_name:
             case R.id.ml_menu_sortby_length:
-                Fragment current = getSupportFragmentManager().findFragmentById(R.id.fragment_placeholder);
                 if (current == null)
                     break;
                 if (current instanceof ISortable)
@@ -627,15 +640,10 @@ public class MainActivity extends SherlockFragmentActivity {
                 break;
             // Refresh
             case R.id.ml_menu_refresh:
-                // TODO: factor this into each fragment
-                if(mCurrentFragment != null && mCurrentFragment.equals("directories")) {
-                    DirectoryViewFragment directoryView = (DirectoryViewFragment) getFragment(mCurrentFragment);
-                    directoryView.refresh();
-                }
-                else if(mCurrentFragment != null && mCurrentFragment.equals("history"))
-                    ((HistoryFragment) getFragment(mCurrentFragment)).refresh();
+                if(current != null && current instanceof IRefreshable)
+                    ((IRefreshable) current).refresh();
                 else
-                    MediaLibrary.getInstance(this).loadMediaItems(this, true);
+                    MediaLibrary.getInstance().loadMediaItems(this, true);
                 break;
             // Restore last playlist
             case R.id.ml_menu_last_playlist:
@@ -660,15 +668,15 @@ public class MainActivity extends SherlockFragmentActivity {
                     break;
                 }
                 /* Toggle the sidebar */
-                if(mMenu.isMenuShowing())
-                    mMenu.showContent();
-                else
-                    mMenu.showMenu();
+                if (mDrawerToggle.onOptionsItemSelected(item)) {
+                    return true;
+                }
                 break;
             case R.id.search_clear_history:
-                MediaDatabase.getInstance(this).clearSearchHistory();
+                MediaDatabase.getInstance().clearSearchHistory();
                 break;
         }
+        mRootContainer.closeDrawer(mListView);
         return super.onOptionsItemSelected(item);
     }
 
@@ -677,7 +685,7 @@ public class MainActivity extends SherlockFragmentActivity {
         super.onActivityResult(requestCode, resultCode, data);
         if (requestCode == ACTIVITY_RESULT_PREFERENCES) {
             if (resultCode == PreferencesActivity.RESULT_RESCAN)
-                MediaLibrary.getInstance(this).loadMediaItems(this, true);
+                MediaLibrary.getInstance().loadMediaItems(this, true);
             else if (resultCode == PreferencesActivity.RESULT_RESTART) {
                 Intent intent = getIntent();
                 finish();
@@ -686,35 +694,18 @@ public class MainActivity extends SherlockFragmentActivity {
         }
     }
 
+    // Note. onKeyDown will not occur while moving within a list
+    @Override
+    public boolean onKeyDown(int keyCode, KeyEvent event) {
+        mFocusedPrior = getCurrentFocus().getId();
+        return super.onKeyDown(keyCode, event);
+    }
+
     private void reloadPreferences() {
         SharedPreferences sharedPrefs = getSharedPreferences("MainActivity", MODE_PRIVATE);
         mCurrentFragment = sharedPrefs.getString("fragment", "video");
     }
 
-    private void showInfoDialog() {
-        final Dialog infoDialog = new Dialog(this, R.style.info_dialog);
-        infoDialog.setContentView(R.layout.info_dialog);
-        Button okButton = (Button) infoDialog.findViewById(R.id.ok);
-        okButton.setOnClickListener(new OnClickListener() {
-            @Override
-            public void onClick(View view) {
-                CheckBox notShowAgain =
-                        (CheckBox) infoDialog.findViewById(R.id.not_show_again);
-                if (notShowAgain.isChecked() && mSettings != null) {
-                    Editor editor = mSettings.edit();
-                    editor.putInt(PREF_SHOW_INFO, mVersionNumber);
-                    editor.commit();
-                }
-                /* Close the dialog */
-                infoDialog.dismiss();
-                /* and finally open the sliding menu if first run */
-                if (mFirstRun)
-                    mMenu.showMenu();
-            }
-        });
-        infoDialog.show();
-    }
-
     /**
      * onClick event from xml
      * @param view
@@ -779,35 +770,29 @@ public class MainActivity extends SherlockFragmentActivity {
         }
     };
 
-    public static void showProgressBar(Context context) {
-        if (context == null)
-            return;
+    public static void showProgressBar() {
         Intent intent = new Intent();
         intent.setAction(ACTION_SHOW_PROGRESSBAR);
-        context.getApplicationContext().sendBroadcast(intent);
+        VLCApplication.getAppContext().sendBroadcast(intent);
     }
 
-    public static void hideProgressBar(Context context) {
-        if (context == null)
-            return;
+    public static void hideProgressBar() {
         Intent intent = new Intent();
         intent.setAction(ACTION_HIDE_PROGRESSBAR);
-        context.getApplicationContext().sendBroadcast(intent);
+        VLCApplication.getAppContext().sendBroadcast(intent);
     }
 
-    public static void sendTextInfo(Context context, String info, int progress, int max) {
-        if (context == null)
-            return;
+    public static void sendTextInfo(String info, int progress, int max) {
         Intent intent = new Intent();
         intent.setAction(ACTION_SHOW_TEXTINFO);
         intent.putExtra("info", info);
         intent.putExtra("progress", progress);
         intent.putExtra("max", max);
-        context.getApplicationContext().sendBroadcast(intent);
+        VLCApplication.getAppContext().sendBroadcast(intent);
     }
 
-    public static void clearTextInfo(Context context) {
-        sendTextInfo(context, null, 0, 100);
+    public static void clearTextInfo() {
+        sendTextInfo(null, 0, 100);
     }
 
     private void onOpenMRL() {
@@ -907,20 +892,69 @@ public class MainActivity extends SherlockFragmentActivity {
                 if (resId != 0)
                     mSlidingPane.setShadowResource(resId);
                 mAudioPlayer.setHeaderVisibilities(false, false, true, true, true);
-                mMenu.setSlidingEnabled(true);
+                mRootContainer.setDrawerLockMode(DrawerLayout.LOCK_MODE_UNLOCKED);
+                removeTipViewIfDisplayed();
+                mAudioPlayer.showAudioPlayerTips();
             }
 
             @Override
             public void onPanelOpenedEntirely() {
                 mSlidingPane.setShadowDrawable(null);
-                mMenu.setSlidingEnabled(true);
+                mRootContainer.setDrawerLockMode(DrawerLayout.LOCK_MODE_UNLOCKED);
             }
 
             @Override
             public void onPanelClosed() {
                 mAudioPlayer.setHeaderVisibilities(true, true, false, false, false);
-                mMenu.setSlidingEnabled(false);
+                mRootContainer.setDrawerLockMode(DrawerLayout.LOCK_MODE_LOCKED_CLOSED);
+                mAudioPlayer.showPlaylistTips();
             }
 
     };
+
+    /**
+     * Show a tip view.
+     * @param layoutId the layout of the tip view
+     * @param settingKey the setting key to check if the view must be displayed or not.
+     */
+    public void showTipViewIfNeeded(final int layoutId, final String settingKey) {
+        if (!mSettings.getBoolean(settingKey, false)) {
+            removeTipViewIfDisplayed();
+            View v = LayoutInflater.from(this).inflate(layoutId, null);
+            mRootContainer.addView(v,
+                    new DrawerLayout.LayoutParams(DrawerLayout.LayoutParams.MATCH_PARENT,
+                    		DrawerLayout.LayoutParams.MATCH_PARENT));
+
+            v.setOnClickListener(new View.OnClickListener() {
+                @Override
+                public void onClick(View v) {
+                    removeTipViewIfDisplayed();
+                }
+            });
+
+            TextView okGotIt = (TextView) v.findViewById(R.id.okgotit_button);
+            okGotIt.setOnClickListener(new View.OnClickListener() {
+                @Override
+                public void onClick(View v) {
+                    removeTipViewIfDisplayed();
+                    SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(MainActivity.this);
+                    Editor editor = settings.edit();
+                    editor.putBoolean(settingKey, true);
+                    editor.commit();
+                }
+            });
+        }
+    }
+
+    /**
+     * Remove the current tip view if there is one displayed.
+     */
+    public void removeTipViewIfDisplayed() {
+        if (mRootContainer.getChildCount() > 2){
+            for (int i = 0 ; i< mRootContainer.getChildCount() ; ++i){
+            if (mRootContainer.getChildAt(i).getId() == R.id.audio_tips)
+                mRootContainer.removeViewAt(i);
+            }
+        }
+    }
 }
