@@ -20,15 +20,16 @@
  *****************************************************************************/
 package org.videolan.vlc.gui;
 
+import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashMap;
 import java.util.List;
 
 import org.videolan.vlc.R;
-import org.videolan.vlc.Util;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.audio.AudioBrowserFragment;
 import org.videolan.vlc.gui.video.VideoGridFragment;
+import org.videolan.vlc.util.Util;
 
 import android.content.Context;
 import android.graphics.Typeface;
@@ -59,6 +60,7 @@ public class SidebarAdapter extends BaseAdapter {
     private Context mContext;
     private LayoutInflater mInflater;
     static final List<SidebarEntry> entries;
+    public static final List<String> sidebarFragments;
     private HashMap<String, Fragment> mFragments;
     private String mCurrentFragmentId;
 
@@ -72,6 +74,11 @@ public class SidebarAdapter extends BaseAdapter {
             //new SidebarEntry( "playlists", R.string.playlists, R.drawable.icon ),
         };
         entries = Arrays.asList(entries2);
+
+        sidebarFragments = new ArrayList<String>();
+        for(SidebarEntry e : entries2) {
+            sidebarFragments.add(e.id);
+        }
     }
 
     public SidebarAdapter(Context context) {
@@ -92,7 +99,7 @@ public class SidebarAdapter extends BaseAdapter {
 
     @Override
     public long getItemId(int position) {
-        return (long)position; // The SidebarEntry list is unique
+        return position; // The SidebarEntry list is unique
     }
 
     @Override
@@ -131,6 +138,7 @@ public class SidebarAdapter extends BaseAdapter {
         if(mFragments.containsKey(id) && mFragments.get(id) != null) {
             return mFragments.get(id);
         }
+
         Fragment f;
         if(id.equals("audio")) {
             f = new AudioBrowserFragment();
