@@ -26,14 +26,14 @@ import java.util.List;
 
 import org.videolan.libvlc.LibVlcUtil;
 import org.videolan.libvlc.Media;
-import org.videolan.vlc.AudioServiceController;
 import org.videolan.vlc.MediaLibrary;
 import org.videolan.vlc.R;
-import org.videolan.vlc.Util;
-import org.videolan.vlc.VlcRunnable;
-import org.videolan.vlc.WeakHandler;
+import org.videolan.vlc.audio.AudioServiceController;
 import org.videolan.vlc.gui.CommonDialogs;
 import org.videolan.vlc.gui.MainActivity;
+import org.videolan.vlc.util.AndroidDevices;
+import org.videolan.vlc.util.VLCRunnable;
+import org.videolan.vlc.util.WeakHandler;
 import org.videolan.vlc.widget.FlingViewGroup;
 import org.videolan.vlc.widget.FlingViewGroup.ViewSwitchListener;
 import org.videolan.vlc.widget.HeaderScrollView;
@@ -44,6 +44,8 @@ import android.os.Build;
 import android.os.Bundle;
 import android.os.Handler;
 import android.os.Message;
+import android.support.v4.app.Fragment;
+import android.support.v7.app.ActionBarActivity;
 import android.view.ContextMenu;
 import android.view.ContextMenu.ContextMenuInfo;
 import android.view.LayoutInflater;
@@ -63,9 +65,7 @@ import android.widget.ListView;
 import android.widget.PopupMenu;
 import android.widget.PopupMenu.OnMenuItemClickListener;
 
-import com.actionbarsherlock.app.SherlockFragment;
-
-public class AudioBrowserFragment extends SherlockFragment {
+public class AudioBrowserFragment extends Fragment {
     public final static String TAG = "VLC/AudioBrowserFragment";
 
     private FlingViewGroup mFlingViewGroup;
@@ -97,7 +97,7 @@ public class AudioBrowserFragment extends SherlockFragment {
 
         mAudioController = AudioServiceController.getInstance();
 
-        mMediaLibrary = MediaLibrary.getInstance(getActivity());
+        mMediaLibrary = MediaLibrary.getInstance();
 
         mSongsAdapter = new AudioBrowserListAdapter(getActivity(), AudioBrowserListAdapter.ITEM_WITH_COVER);
         mArtistsAdapter = new AudioBrowserListAdapter(getActivity(), AudioBrowserListAdapter.ITEM_WITH_COVER);
@@ -113,7 +113,7 @@ public class AudioBrowserFragment extends SherlockFragment {
     @Override
     public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
     {
-        getSherlockActivity().getSupportActionBar().setTitle(R.string.audio);
+        ((ActionBarActivity) getActivity()).getSupportActionBar().setTitle(R.string.audio);
 
         View v = inflater.inflate(R.layout.audio_browser, container, false);
 
@@ -227,7 +227,7 @@ public class AudioBrowserFragment extends SherlockFragment {
             MenuItem play = menu.findItem(R.id.audio_list_browser_play);
             play.setVisible(true);
         }
-        if (!Util.isPhone())
+        if (!AndroidDevices.isPhone())
             menu.setGroupVisible(R.id.phone_only, false);
     }
 
@@ -264,7 +264,7 @@ public class AudioBrowserFragment extends SherlockFragment {
             AlertDialog alertDialog = CommonDialogs.deleteMedia(
                     getActivity(),
                     mSongsAdapter.getLocations(groupPosition).get(0),
-                    new VlcRunnable(mSongsAdapter.getItem(groupPosition)) {
+                    new VLCRunnable(mSongsAdapter.getItem(groupPosition)) {
                         @Override
                         public void run(Object o) {
                             AudioBrowserListAdapter.ListItem listItem = (AudioBrowserListAdapter.ListItem)o;
@@ -298,7 +298,7 @@ public class AudioBrowserFragment extends SherlockFragment {
                     medias = mArtistsAdapter.getLocations(groupPosition);
                     break;
                 case MODE_ALBUM:
-                    medias = mArtistsAdapter.getLocations(groupPosition);
+                    medias = mAlbumsAdapter.getLocations(groupPosition);
                     break;
                 case MODE_GENRE:
                     medias = mGenresAdapter.getLocations(groupPosition);
@@ -346,6 +346,9 @@ public class AudioBrowserFragment extends SherlockFragment {
 
         @Override
         public void onTouchClick() {}
+
+        @Override
+        public void onBackSwitched() {}
     };
 
     /**
@@ -372,7 +375,7 @@ public class AudioBrowserFragment extends SherlockFragment {
     };
 
     private void updateLists() {
-        List<Media> audioList = MediaLibrary.getInstance(getActivity()).getAudioItems();
+        List<Media> audioList = MediaLibrary.getInstance().getAudioItems();
 
         if (audioList.isEmpty())
             mEmptyView.setVisibility(View.VISIBLE);
@@ -389,32 +392,42 @@ public class AudioBrowserFragment extends SherlockFragment {
             Media media = audioList.get(i);
             mSongsAdapter.add(media.getTitle(), media.getArtist(), media);
         }
+        mSongsAdapter.addScrollSections();
 
         Collections.sort(audioList, MediaComparators.byArtist);
         for (int i = 0; i < audioList.size(); i++) {
             Media media = audioList.get(i);
             mArtistsAdapter.add(media.getArtist(), null, media);
         }
-        mArtistsAdapter.addLeterSeparators();
+        mArtistsAdapter.addLetterSeparators();
 
         Collections.sort(audioList, MediaComparators.byAlbum);
         for (int i = 0; i < audioList.size(); i++) {
             Media media = audioList.get(i);
             mAlbumsAdapter.add(media.getAlbum(), media.getArtist(), media);
         }
-        mAlbumsAdapter.addLeterSeparators();
+        mAlbumsAdapter.addLetterSeparators();
 
         Collections.sort(audioList, MediaComparators.byGenre);
         for (int i = 0; i < audioList.size(); i++) {
             Media media = audioList.get(i);
             mGenresAdapter.add(media.getGenre(), null, media);
         }
-        mGenresAdapter.addLeterSeparators();
+        mGenresAdapter.addLetterSeparators();
 
         mSongsAdapter.notifyDataSetChanged();
         mArtistsAdapter.notifyDataSetChanged();
         mAlbumsAdapter.notifyDataSetChanged();
         mGenresAdapter.notifyDataSetChanged();
+        // Refresh the fast scroll data, since SectionIndexer doesn't respect notifyDataSetChanged
+        int[] lists = { R.id.songs_list, R.id.artists_list, R.id.albums_list, R.id.genres_list };
+        if(getView() != null) {
+            for(int r : lists) {
+                ListView l = (ListView)getView().findViewById(r);
+                l.setFastScrollEnabled(false);
+                l.setFastScrollEnabled(true);
+            }
+        }
     }
 
     AudioBrowserListAdapter.ContextPopupMenuListener mContextPopupMenuListener
