@@ -1,7 +1,7 @@
 /*****************************************************************************
- * AudioMiniPlayer.java
+ * AudioPlayer.java
  *****************************************************************************
- * Copyright © 2011-2012 VLC authors and VideoLAN
+ * Copyright © 2011-2014 VLC authors and VideoLAN
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,20 +27,23 @@ import android.content.Intent;
 
 import org.videolan.libvlc.LibVLC;
 import org.videolan.libvlc.Media;
-import org.videolan.vlc.AudioServiceController;
 import org.videolan.vlc.R;
-import org.videolan.vlc.RepeatType;
-import org.videolan.vlc.Util;
+import org.videolan.vlc.audio.AudioServiceController;
+import org.videolan.vlc.audio.RepeatType;
 import org.videolan.vlc.gui.CommonDialogs;
 import org.videolan.vlc.gui.MainActivity;
 import org.videolan.vlc.gui.CommonDialogs.MenuType;
 import org.videolan.vlc.gui.audio.widget.CoverMediaSwitcher;
 import org.videolan.vlc.gui.audio.widget.HeaderMediaSwitcher;
 import org.videolan.vlc.interfaces.IAudioPlayer;
+import org.videolan.vlc.util.Strings;
+import org.videolan.vlc.util.Util;
 import org.videolan.vlc.widget.AudioMediaSwitcher.AudioMediaSwitcherListener;
 
 import android.media.AudioManager;
 import android.os.Bundle;
+import android.os.Handler;
+import android.preference.PreferenceManager;
 import android.support.v4.app.Fragment;
 import android.support.v4.app.FragmentActivity;
 import android.util.Log;
@@ -48,6 +51,7 @@ import android.view.ContextMenu;
 import android.view.LayoutInflater;
 import android.view.MenuInflater;
 import android.view.MenuItem;
+import android.view.MotionEvent;
 import android.view.View;
 import android.view.ViewGroup;
 import android.view.ContextMenu.ContextMenuInfo;
@@ -63,7 +67,7 @@ import android.widget.AdapterView.OnItemClickListener;
 import android.widget.SeekBar.OnSeekBarChangeListener;
 
 public class AudioPlayer extends Fragment implements IAudioPlayer {
-    public static final String TAG = "VLC/AudioMiniPlayer";
+    public static final String TAG = "VLC/AudioPlayer";
 
     private ProgressBar mProgressBar;
     private HeaderMediaSwitcher mHeaderMediaSwitcher;
@@ -86,6 +90,7 @@ public class AudioPlayer extends Fragment implements IAudioPlayer {
 
     private AudioServiceController mAudioController;
     private boolean mShowRemainingTime = false;
+    private boolean mPreviewingSeek = false;
 
     private AudioPlaylistAdapter mSongsListAdapter;
 
@@ -95,6 +100,10 @@ public class AudioPlayer extends Fragment implements IAudioPlayer {
     private boolean mProgressBarVisible;
     private boolean mHeaderTimeVisible;
 
+    // Tips
+    private static final String PREF_PLAYLIST_TIPS_SHOWN = "playlist_tips_shown";
+    private static final String PREF_AUDIOPLAYER_TIPS_SHOWN = "audioplayer_tips_shown";
+
     @Override
     public void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
@@ -174,18 +183,18 @@ public class AudioPlayer extends Fragment implements IAudioPlayer {
                 return true;
             }
         });
-        mNext.setOnClickListener(new View.OnClickListener() {
-            @Override
-            public void onClick(View v) {
-                onNextClick(v);
-            }
-        });
-        mPrevious.setOnClickListener(new View.OnClickListener() {
-            @Override
-            public void onClick(View v) {
-                onPreviousClick(v);
-            }
-        });
+        boolean blackTheme = PreferenceManager.getDefaultSharedPreferences(
+                getActivity()).getBoolean("enable_black_theme", false);
+        mNext.setOnTouchListener(new LongSeekListener(true,
+                blackTheme ? R.drawable.ic_next_normal_w
+                        : R.drawable.ic_next_normal,
+                blackTheme ? R.drawable.ic_next_pressed_w
+                        : R.drawable.ic_next_pressed));
+        mPrevious.setOnTouchListener(new LongSeekListener(false,
+                blackTheme ? R.drawable.ic_previous_normal_w
+                        : R.drawable.ic_previous_normal,
+                blackTheme ? R.drawable.ic_previous_pressed_w
+                        : R.drawable.ic_previous_pressed));
         mShuffle.setOnClickListener(new View.OnClickListener() {
             @Override
             public void onClick(View v) {
@@ -232,7 +241,7 @@ public class AudioPlayer extends Fragment implements IAudioPlayer {
         });
         mSongsList.setOnItemDraggedListener(new AudioPlaylistView.OnItemDraggedListener() {
             @Override
-            public void OnItemDradded(int positionStart, int positionEnd) {
+            public void onItemDragged(int positionStart, int positionEnd) {
                 mAudioController.moveItem(positionStart, positionEnd);
             }
         });
@@ -360,13 +369,17 @@ public class AudioPlayer extends Fragment implements IAudioPlayer {
     public synchronized void updateProgress() {
         int time = mAudioController.getTime();
         int length = mAudioController.getLength();
-        mTime.setText(Util.millisToString(mShowRemainingTime ? time-length : time));
-        mHeaderTime.setText(Util.millisToString(time));
-        mLength.setText(Util.millisToString(length));
+
+        mHeaderTime.setText(Strings.millisToString(time));
+        mLength.setText(Strings.millisToString(length));
         mTimeline.setMax(length);
-        mTimeline.setProgress(time);
         mProgressBar.setMax(length);
-        mProgressBar.setProgress(time);
+
+        if(!mPreviewingSeek) {
+            mTime.setText(Strings.millisToString(mShowRemainingTime ? time-length : time));
+            mTimeline.setProgress(time);
+            mProgressBar.setProgress(time);
+        }
     }
 
     private void updateList() {
@@ -412,8 +425,8 @@ public class AudioPlayer extends Fragment implements IAudioPlayer {
         public void onProgressChanged(SeekBar sb, int prog, boolean fromUser) {
             if (fromUser) {
                 mAudioController.setTime(prog);
-                mTime.setText(Util.millisToString(mShowRemainingTime ? prog-mAudioController.getLength() : prog));
-                mHeaderTime.setText(Util.millisToString(prog));
+                mTime.setText(Strings.millisToString(mShowRemainingTime ? prog-mAudioController.getLength() : prog));
+                mHeaderTime.setText(Strings.millisToString(prog));
         }
     }
     };
@@ -470,12 +483,14 @@ public class AudioPlayer extends Fragment implements IAudioPlayer {
 
     public void show() {
         MainActivity activity = (MainActivity)getActivity();
-        activity.showAudioPlayer();
+        if (activity != null)
+            activity.showAudioPlayer();
     }
 
     public void hide() {
         MainActivity activity = (MainActivity)getActivity();
-        activity.hideAudioPlayer();
+        if (activity != null)
+            activity.hideAudioPlayer();
     }
 
     /**
@@ -563,4 +578,100 @@ public class AudioPlayer extends Fragment implements IAudioPlayer {
         public void onTouchClick() {}
     };
 
+    class LongSeekListener implements View.OnTouchListener {
+        boolean forward;
+        int normal, pressed;
+        long length;
+
+        public LongSeekListener(boolean forwards, int normalRes, int pressedRes) {
+            this.forward = forwards;
+            this.normal = normalRes;
+            this.pressed = pressedRes;
+            this.length = -1;
+        }
+
+        int possibleSeek;
+        boolean vibrated;
+        Runnable seekRunnable = new Runnable() {
+            @Override
+            public void run() {
+                if(!vibrated) {
+                    ((android.os.Vibrator) AudioPlayer.this.getActivity()
+                            .getSystemService(Context.VIBRATOR_SERVICE))
+                            .vibrate(80);
+                    ;
+                    vibrated = true;
+                }
+
+                if(forward) {
+                    if(length <= 0 || possibleSeek < length)
+                        possibleSeek += 4000;
+                } else {
+                    if(possibleSeek > 4000)
+                        possibleSeek -= 4000;
+                    else if(possibleSeek <= 4000)
+                        possibleSeek = 0;
+                }
+
+                mTime.setText(Strings.millisToString(mShowRemainingTime ? possibleSeek-length : possibleSeek));
+                mTimeline.setProgress(possibleSeek);
+                mProgressBar.setProgress(possibleSeek);
+                h.postDelayed(seekRunnable, 50);
+            }
+        };
+        Handler h = new Handler();
+        @Override
+        public boolean onTouch(View v, MotionEvent event) {
+            switch(event.getAction()) {
+            case MotionEvent.ACTION_DOWN:
+                (forward ? mNext : mPrevious).setImageResource(this.pressed);
+
+                possibleSeek = mAudioController.getTime();
+                mPreviewingSeek = true;
+                vibrated = false;
+                length = mAudioController.getLength();
+
+                h.postDelayed(seekRunnable, 1000);
+                return true;
+
+            case MotionEvent.ACTION_UP:
+                (forward ? mNext : mPrevious).setImageResource(this.normal);
+                h.removeCallbacks(seekRunnable);
+                mPreviewingSeek = false;
+
+                if(event.getEventTime()-event.getDownTime() < 1000) {
+                    if(forward)
+                        onNextClick(v);
+                    else
+                        onPreviousClick(v);
+                } else {
+                    if(forward) {
+                        if(possibleSeek < mAudioController.getLength())
+                            mAudioController.setTime(possibleSeek);
+                        else
+                            onNextClick(v);
+                    } else {
+                        if(possibleSeek > 0)
+                            mAudioController.setTime(possibleSeek);
+                        else
+                            onPreviousClick(v);
+                    }
+                }
+                return true;
+            }
+            return false;
+        }
+    }
+
+    public void showPlaylistTips() {
+        MainActivity activity = (MainActivity)getActivity();
+        if(activity != null)
+            activity.showTipViewIfNeeded(R.layout.audio_playlist_tips, PREF_PLAYLIST_TIPS_SHOWN);
+    }
+
+    public void showAudioPlayerTips() {
+        MainActivity activity = (MainActivity)getActivity();
+        if(activity != null)
+            activity.showTipViewIfNeeded(R.layout.audio_player_tips, PREF_AUDIOPLAYER_TIPS_SHOWN);
+    }
 }
