@@ -22,18 +22,19 @@ package org.videolan.vlc.gui.audio;
 import org.videolan.libvlc.LibVLC;
 import org.videolan.libvlc.LibVlcException;
 import org.videolan.vlc.R;
-import org.videolan.vlc.Util;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.interfaces.OnEqualizerBarChangeListener;
+import org.videolan.vlc.util.Preferences;
+import org.videolan.vlc.util.VLCInstance;
 import org.videolan.vlc.widget.EqualizerBar;
 
-import com.actionbarsherlock.app.SherlockFragment;
-
 import android.content.Context;
 import android.content.SharedPreferences;
 import android.content.res.Configuration;
 import android.os.Bundle;
 import android.preference.PreferenceManager;
+import android.support.v4.app.Fragment;
+import android.support.v7.app.ActionBarActivity;
 import android.view.LayoutInflater;
 import android.view.View;
 import android.view.ViewGroup;
@@ -49,7 +50,7 @@ import android.widget.SeekBar.OnSeekBarChangeListener;
 import android.widget.Spinner;
 import android.widget.ToggleButton;
 
-public class EqualizerFragment extends SherlockFragment {
+public class EqualizerFragment extends Fragment {
 
     public final static String TAG = "VLC/EqualizerFragment";
     private ToggleButton button;
@@ -66,7 +67,7 @@ public class EqualizerFragment extends SherlockFragment {
     @Override
     public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
     {
-        getSherlockActivity().getSupportActionBar().setTitle(getResources().getString(R.string.equalizer));
+        ((ActionBarActivity) getActivity()).getSupportActionBar().setTitle(getResources().getString(R.string.equalizer));
 
         super.onCreateView(inflater, container, savedInstanceState);
         View v = inflater.inflate(R.layout.equalizer, container, false);
@@ -107,11 +108,11 @@ public class EqualizerFragment extends SherlockFragment {
         float[] bands = null;
         String[] presets = null;
         try {
-            libVlc = Util.getLibVlcInstance();
+            libVlc = VLCInstance.getLibVlcInstance();
             bands = libVlc.getBands();
             presets = libVlc.getPresets();
             if (equalizer == null)
-                equalizer = Util.getFloatArray(preferences, "equalizer_values");
+                equalizer = Preferences.getFloatArray(preferences, "equalizer_values");
             if (equalizer == null)
                 equalizer = new float[bands.length + 1];
         } catch (LibVlcException e) {
@@ -175,7 +176,7 @@ public class EqualizerFragment extends SherlockFragment {
         SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext());
         SharedPreferences.Editor editor = preferences.edit();
         editor.putBoolean("equalizer_enabled", button.isChecked());
-        Util.putFloatArray(editor, "equalizer_values", equalizer);
+        Preferences.putFloatArray(editor, "equalizer_values", equalizer);
         editor.putInt("equalizer_preset", equalizer_presets.getSelectedItemPosition());
         editor.commit();
     }
